import os
from contextlib import contextmanager


@contextmanager
def environment_append(env_vars):
    unset_vars = []
    for key in env_vars.keys():
        if env_vars[key] is None:
            unset_vars.append(key)
    for var in unset_vars:
        env_vars.pop(var, None)
    for name, value in env_vars.items():
        if isinstance(value, list):
            env_vars[name] = os.pathsep.join(value)
            old = os.environ.get(name)
            if old:
                env_vars[name] += os.pathsep + old
    if env_vars or unset_vars:
        old_env = dict(os.environ)
        os.environ.update(env_vars)
        for var in unset_vars:
            os.environ.pop(var, None)
        try:
            yield
        finally:
            os.environ.clear()
            os.environ.update(old_env)
    else:
        yield




import argparse
import os

from conan_index_tools.new_recipe import new_recipe
from conan_index_tools.runner import run_create


def run(*args):
    parser = argparse.ArgumentParser(description="", prog="cit")
    subparsers = parser.add_subparsers(dest='subcommand', help='sub-command help')
    subparsers.required = True

    # 'NEW' COMMAND
    new_parser = subparsers.add_parser('new', help='Add a new package version')
    new_parser.add_argument("index_path", help='Path to a clone of the conan center index fork')
    new_parser.add_argument("-n", "--name", help='Name of the package')
    new_parser.add_argument("-v", "--version", help='Version of the package')
    new_parser.add_argument("-f", "--folder", help='Folder where the recipe will be')
    new_parser.add_argument("-s", "--url", help='URL to a zip with the sources')

    # 'SETUP' COMMAND
    setup_parser = subparsers.add_parser('setup', help='Enable the conan-center hook')
    setup_parser.add_argument("-c", "--clean", action='store_true', default=False,
                              help='Disable the conan-center hook')

    # 'RUN' COMMAND
    run_parser = subparsers.add_parser('run', help='Runs the conan create of a reference')
    run_parser.add_argument("index_path", help='Path to a clone of the conan center index fork')
    run_parser.add_argument("ref", help='package_name/version. e.g: lib/1.0')
    run_parser.add_argument("-u", "--profile_url", help='A URL to the profile to be downloaded')
    run_parser.add_argument("-p", "--profile", help='Profile name or path')
    try:
        args = parser.parse_args(*args)
        if args.subcommand == "new":
            return new_recipe(args.index_path, args.name, args.version, args.folder, args.url)
        elif args.subcommand == "setup":
            if not args.clean:
                hooks_url = "https://github.com/lasote/hooks.git"
                os.system("conan config install {} -sf hooks -tf hooks".format(hooks_url))
                os.system("conan config set hooks.conan-center")
                print("Hook installed")
            else:
                os.system("conan config rm hooks.conan-center")
                print("Hook deactivated")
        elif args.subcommand == "run":
            run_create(args.index_path, args.ref, args.profile, args.profile_url)
    except KeyboardInterrupt:
        print("\nKilled!")
    except Exception as exc:
        print("Error: {}".format(exc))
        exit(-1)





import os
import yaml

USER_CHANNEL = ""


class IndexStorage(object):

    def __init__(self, base_path):
        self.base_path = base_path
        if not os.path.exists(os.path.join(self.base_path, "recipes")):
            raise Exception("Index repository not found at: {}".format(base_path))

    @property
    def recipes_folder(self):
        return os.path.join(self.base_path, "recipes")

    def real_recipe_folder(self, name, version):
        best = version
        if os.path.exists(self.config_yml_path(name)):
            data = self.load_config_yml(name)
            if "versions" in data and version in data["versions"] and \
                    "folder" in data["versions"][version]:
                 best = data["versions"][version]["folder"]
        return os.path.abspath(self.recipe_folder(name, best))

    def recipe_folder(self, name, folder):
        return os.path.join(self.package_folder(name), folder)

    def conanfile_path(self, name, folder):
        return os.path.join(self.package_folder(name), folder, "conanfile.py")

    def package_folder(self, name):
        return os.path.join(self.recipes_folder, name)

    def config_yml_path(self, name):
        return os.path.join(self.recipes_folder, name, "config.yml")

    @property
    def package_names(self):
        return [n for n in os.listdir(self.recipes_folder)
                if os.path.isdir(os.path.join(self.recipes_folder, n))]

    def folders(self, name):
        try:
            return [n for n in os.listdir(self.package_folder(name))
                    if os.path.isdir(os.path.join(self.package_folder(name), n))]
        except IOError:
            return []

    def versions(self, name):
        if os.path.exists(self.config_yml_path(name)):
            data = self.load_config_yml(name)
            if "versions" in data:
                return data["versions"].keys()
        else:
            if not os.path.exists(self.package_folder(name)):
                return []
            return [n for n in os.listdir(self.package_folder(name))
                    if os.path.isdir(os.path.join(self.recipes_folder, name, n))]

        return ["1.0"]

    def add_new_version(self, name, version, folder):
        element = {"folders": folder}

        recipe_folder = self.recipe_folder(name, folder)
        if not os.path.exists(recipe_folder):
            os.makedirs(recipe_folder)

        if os.path.exists(self.config_yml_path(name)):
            data = self.load_config_yml(name)
            if "versions" not in data:
                data["versions"] = {}
            data["versions"][version] = element

            with open(self.config_yml_path(name), "w") as fl:
                contents = yaml.dump(data)
                fl.write(contents)
        else:  # If folder is "all" for example, we suppose we want to support multiple
            if version != folder:
                with open(self.config_yml_path(name), "w") as fl:
                    data = {"versions": [element]}
                    contents = yaml.dump(data)
                    fl.write(contents)

        print("Added new version {}/{}".format(name, version))

    def generate_recipe_for_version(self, name, version, folder, header_only=False):
        recipe_folder = self.recipe_folder(name, folder)
        cf_path = os.path.join(recipe_folder, "conanfile.py")
        if not os.path.exists(cf_path):
            import subprocess
            # FIXME: Use a template for c3i with "-m" when it allow to pass an abs path
            cmd = "conan new {}/{}@no/no -t{}".format(name, version, " -i" if header_only else "")
            p = subprocess.Popen(cmd.split(" "), cwd=recipe_folder, stdout=subprocess.DEVNULL)
            code = p.wait()
            if code != 0:
                raise Exception("conan new command failed")

            # Override the conanfile
            tname = "layout.txt" if not header_only else "header_only_layout.txt"
            template = os.path.join(os.path.dirname(__file__), "assets", tname)
            with open(template) as fn:
                contents = fn.read()
                ret = contents.format(name=name, version=version)
            with open(cf_path, "w") as fn:
                fn.write(ret)

    def add_entry_to_conandata_yml(self, name, version, folder, url, the_hash, hash_type):
        # Create conandata.yml
        recipe_folder = os.path.join(self.recipes_folder, name, folder)
        entry = {"url": url, hash_type: the_hash}
        data = {"sources": {version: entry}}
        data_yml_path = os.path.join(recipe_folder, "conandata.yml")
        yml_file = os.path.exists(data_yml_path)
        if yml_file:
            with open(data_yml_path) as fl:
                contents = fl.read(yml_file)
                data = yaml.safe_load(contents)
                data["sources"][version].append(entry)

        with open(data_yml_path, "w") as fl:
            contents = yaml.dump(data)
            fl.write(contents)

    def load_config_yml(self, name):
        with open(self.config_yml_path(name)) as fl:
            contents = fl.read()
            return yaml.safe_load(contents)

    def save_config_yml(self, name, data):
        contents = yaml.safe_dump(data)
        with open(self.config_yml_path(name), "w") as fl:
            fl.write(contents)




import os
import platform
import tempfile
from conans.util.config_parser import ConfigParser

import requests

from conan_index_tools.index_storage import IndexStorage, USER_CHANNEL
from conan_index_tools.tools import environment_append


def get_docker_image_name(settings):
    docker_image = "conanio/{}{}".format(settings["compiler"],
                                         settings["compiler.version"].replace(".", ""))
    return docker_image


def _get_settings(profile_path):
    with open(profile_path) as fp:
        contents = fp.read()
    doc = ConfigParser(contents, allowed_fields=["build_requires", "settings", "env",
                                                 "scopes", "options"])
    settings = {}
    for line in doc.settings.splitlines():
        chunks = line.split("=")
        settings[chunks[0]] = chunks[1]
    return settings


def get_profile_path(profile_name):
    home = os.environ.get("CONAN_USER_HOME", os.path.expanduser("~/.conan"))
    return os.path.abspath(os.path.expanduser(os.path.join(home, "profiles", profile_name)))


def run_create(index_path, ref, profile_name, profile_url):

    if profile_name and profile_url:
        raise Exception("Do not specify profile and profile_url")

    if profile_url:
        try:
            r = requests.get(profile_url, allow_redirects=True)
            profile_text = r.content
            tmp_path = tempfile.mkdtemp()
            profile_path = os.path.join(tmp_path, "profile.txt")
            with open(profile_path, "w") as fd:
                try:
                    profile_text = profile_text.decode()
                except:
                    pass
                fd.write(profile_text)
        except Exception as e:
            print("Error downloading the file: {}".format(e))
            exit(-1)
    elif profile_name:
        profile_path = get_profile_path(profile_name)
    else:
        profile_path = get_profile_path("default")

    cur_os = {"Darwin": "Macos"}.get(platform.system(), platform.system())
    storage = IndexStorage(index_path)
    settings = _get_settings(profile_path)

    if settings["os_build"] != "Linux" and cur_os != settings["os_build"]:
        raise Exception("Cannot run a build of {} in this system".format(settings["os_build"]))

    recipe_folder = storage.real_recipe_folder(*ref.split("/"))
    print("Running at: {}".format(recipe_folder))

    my_env = os.environ.copy()

    if ((settings and settings["os_build"] == "Linux") or
       (not settings and cur_os == "Linux")) and docker_present():
        docker_image = get_docker_image_name(settings)
        create = "conan profile list && conan create recipe {}@{}".format(ref, USER_CHANNEL)
        inside = "pip uninstall -y conan-package-tools && " \
                 "pip install conan --upgrade &&" \
                 "rm -rf ./recipe/test_package/build && " + create
        cmd = 'docker run --rm -v{}:/home/conan/recipe -v{}:/home/conan/profiles ' \
              '-e CONAN_HOOK_ERROR_LEVEL=40 -e CONAN_USER_HOME=/home/conan/ ' \
              '{} /bin/bash -c "{}"'.format(recipe_folder, os.path.dirname(profile_path),
                                            docker_image, inside)
    else:
        cmd = "conan create . {}@{} --profile {}".format(ref, profile_path, USER_CHANNEL)
        my_env["CONAN_HOOK_ERROR_LEVEL"] = "40"

    print("---------------------  RUNNING  ---------------------\n{}\n".format(cmd))
    with environment_append(my_env):
        tmp = os.getcwd()
        os.chdir(recipe_folder)
        code = os.system(cmd)
        os.chdir(tmp)
        if code != 0:
            raise Exception("The command '{}' failed! ".format(cmd))


def docker_present():
    return os.system("which docker") == 0



import hashlib
import os
from builtins import input
import requests

from conan_index_tools.index_storage import IndexStorage


def yes_or_no(question):
    while "the answer is invalid":
        reply = str(input(question+' (y/n): ')).lower().strip()
        if reply[0] == 'y':
            return True
        if reply[0] == 'n':
            return False


def command_new_version(storage, name, version, folder, url):
    version = version or input("Enter the version: ")
    cur_versions = storage.versions(name)
    if version in cur_versions:
        print("The version already exists!")
        exit(-1)

    folders = storage.folders(name)
    if folders:
        print("Current recipe folders: {}".format(", ".join(folders)))

    folder = folder or input("Enter the folder name containing the recipe "
                             "(Default {}) ".format(version))
    if not folder:
        folder = version
    storage.add_new_version(name, version, folder)

    if not os.path.exists(storage.conanfile_path(name, folder)):
        create_recipe = yes_or_no("Do you want to create an empty recipe?")
        if create_recipe:
            header_only = yes_or_no("Is it a header only recipe?")
            storage.generate_recipe_for_version(name, version, folder, header_only)

    while True:
        url = url or input("Enter the sources URL for the {} version: ".format(version)).strip()
        print("Calculating SHA256...")
        try:
            r = requests.get(url, allow_redirects=True)
        except Exception as e:
            url = None
            print("Error downloading the file: {}".format(e))
            if yes_or_no("Exit?"):
                return
        else:
            the_hash = hashlib.sha256(r.content).hexdigest()
            storage.add_entry_to_conandata_yml(name, version, folder, url, the_hash, "sha256")
            print("\nCreated version files!\n")
            break

    return storage.recipe_folder(name, folder), version


def new_recipe(base_path, name, version, folder, url):

    storage = IndexStorage(os.path.abspath(base_path))
    name = name or input("Introduce the package name (lowercase). e.g: openssl: ")
    if name.lower() != name:
        print("The package name should be lowercase!")
        exit(-1)
    if name in storage.package_names:
        versions = storage.versions(name)
        print("The package already exists with the "
              "following versions: {}".format(", ".join(versions)))

    recipe_folder, version = command_new_version(storage, name, version, folder, url)

    print("""All done!
    
- Edit your conanfile.py here: '{}'
- Create or modify a 'test_package' folder with a small program testing your package. 
  More info here: https://docs.conan.io/en/latest/creating_packages/getting_started.html
- You can test your recipe running: 
     $ cit run . {}/{} [--profile XXX] [--profile_url YYY]
""".format(recipe_folder, name, version))
